//******************************************************************		
//*** 															 ***
//*** Author :  Alex Brussee									 ***
//*** 			xxlex@hotmail.com					9/07/2003 	 ***
//*** 															 ***
//******************************************************************		
//***                                                            ***
//*** "PathCanvas.java" draws the path of the robot using        ***
//*** path for x,y,phi positions at [t]                          ***
//***                                                            ***
//***                                                            ***
//******************************************************************	

import java.awt.*;
import java.applet.Applet;

public class Formulas
{
	private int Samples,
				Sample_Max;	
				
	public Formulas(int S, int S_Max)
	{
		Samples = S;
		Sample_Max = S_Max;
	}
	
	private double RCRC(double Tau1, double Tau2, double Amp, double T)
	{
		double result;
		Tau1 = Tau1/100;
		Tau2 = Tau2/100;
		
		result = (Amp * ( (1/(Tau1-Tau2)) * Math.exp(-( T/150)/Tau1) - (1/(Tau1-Tau2)) * Math.exp(-( (T) /150)/Tau2)));
		
		return result;
	}
	
	private double Sinus(double Tau1, double Width, double Amp, double T)
	{		
		return((Amp + Amp * Math.sin(Math.PI*(2 * T / Samples - 0.5))) / 4); // division by 4 to shorten resulting path
	}
	
	private double Gauss(double Tau1, double Tau2, double Sigma, double T)
	{
		double 	result,
				Mu = 400; // make sure there is a reasonable amount of Left handed stuff of the formula
		
		Sigma += Sigma;		
				
		result = (10000/(Sigma * Math.sqrt(2 * Math.PI)) * Math.exp(- ((T - Mu)*(T - Mu)) / (2 * Sigma * Sigma)));
			
		return result;
	}

		
	public double Next_Steer(String Formula_Str, int Tau1, int Tau2, int Amp, int i)
	{
		double result = 0;
		
		if(Formula_Str == "RcRcfilter")  { result =  RCRC((double)Tau1, (double)Tau2, (double)Amp, (double)i); } 
		else if(Formula_Str == "Sinus") { result = Sinus((double)Tau1, (double)Tau2, (double)Amp, (double)i); } 
		else if(Formula_Str == "Gauss")  { result = Gauss((double)Tau1, (double)Tau2, (double)Amp, (double)i); }
			
		return (result);
	}
	
	private double[] Carthesian(double X, double Y, double New_ValX, double New_ValY, double Phi)
	{ 
		double result[] = new double[3];
		
		result[0] = X + (New_ValX / 2);
		result[1] = Y + (New_ValY / 2);
		
		if(New_ValX == 0)
		{
			result[2] = Math.PI / 2;
		} else if (New_ValY == 0)
		{
			if(New_ValX == 0) result[2] = Phi;
			else result[2] = 0;
		} else
		{
		 	result[2] = Math.atan(New_ValY/New_ValX);
	 	}
		
		return result;
	}
	
	private double[] Speed(double X, double Y, double New_ValX, double New_ValY, double Phi)
	{ 
		double result[] = new double[3];
		double Axis = 1000;
		
		if(New_ValX == New_ValY)
		{	// no rotation, Phi stays the same
			result[0] = X + New_ValX * Math.cos(Phi);
			result[1] = Y + New_ValY * Math.sin(Phi);		
			result[2] = Phi;
		} else
		{
			// here the robot's trajectory for the timeslice is calculated
			double Phi_new = Phi + ((New_ValX - New_ValY)/Axis);
			double C = (Axis*(New_ValX + New_ValY))/(2*(New_ValX - New_ValY));
		
			// A "circular path" can be calculated
			result[0] = X + C * (Math.sin(Phi_new) - Math.sin(Phi)); // here we need the new and old Phi
			result[1] = Y - C * (Math.cos(Phi_new) - Math.cos(Phi)); // hence the birth of Phi_new
				// new position and orientation of robot is old one for next iteration
			result[2] = Phi_new;
		}		
		return result;
	}
	
	private double[] Energy(double X, double Y, double New_ValX, double New_ValY, double Phi)
	{ 
		double result[] = new double[3];
		double Axis = 1000;
		
		New_ValX = Math.sqrt(New_ValX) * 6;
		New_ValY = Math.sqrt(New_ValY) * 6;
		
		if(New_ValX == New_ValY)
		{	// no rotation, Phi stays the same
			result[0] = X + New_ValX * Math.cos(Phi);
			result[1] = Y + New_ValY * Math.sin(Phi);		
			result[2] = Phi;
		} else
		{
			// here the robot's trajectory for the timeslice is calculated
			double Phi_new = Phi + ((New_ValX - New_ValY)/Axis);
			double C = (Axis*(New_ValX + New_ValY))/(2*(New_ValX - New_ValY));
		
			// A "circular path" can be calculated
			result[0] = X + C * (Math.sin(Phi_new) - Math.sin(Phi)); // here we need the new and old Phi
			result[1] = Y - C * (Math.cos(Phi_new) - Math.cos(Phi)); // hence the birth of Phi_new
				// new position and orientation of robot is old one for next iteration
			result[2] = Phi_new;
		}		
		return result;
	}
	
	public double[] Next_Path(String Formula_Str, double X, double Y, double New_ValX, double New_ValY, double Phi)
	{
		double result[] = new double[3];
		
		if(Formula_Str == "Carthesian")	{ result = Carthesian(	X,  Y,  New_ValX,  New_ValY,  Phi); }
		else if(Formula_Str == "Speed")	{ result = Speed(	 	X,  Y,  New_ValX,  New_ValY,  Phi); }
		else if(Formula_Str == "Energy")	{ result = Energy(	 	X,  Y,  New_ValX,  New_ValY,  Phi); }
			
		return (result);
	}
	
	
	
}