//******************************************************************		
//*** 															 ***
//*** Author :  Alex Brussee									 ***
//*** 			xxlex@hotmail.com					31/07/2003 	 ***
//*** 															 ***
//******************************************************************	
//****                                                          ****
//******************************************************************	
//*** 															 ***
//*** Java applet "Neural.java", shows how a model of a muscle 	 ***
//*** can be used to simulate movement. Differential motor		 ***
//*** control for a robot is the intented use					 ***
//*** 															 ***
//******************************************************************		

import java.awt.*;
import java.applet.Applet;
import java.awt.event.*;
import javax.swing.*;    

public class Path extends Applet implements ActionListener, AdjustmentListener, ItemListener
{

//******************************************************************	
//**** initialisation of values used in applet                  ****
//******************************************************************	
	private Timer timer;

	public	int Amp_Max = 200,
				Sample_Size = 800,
				Sample_Max = 2000,
		  		RC_Max = 200,
		  		Trans_Max = 200,
		  		Time_Max = 1200;
	
	// definition of choices and their initial string values	  			  	
	public String 	Formula_Str = "RcRcfilter", // or "Sinus" or "Gauss"
	 				Steering_Str = "Speed", // or "Carthesian" or "Energy"
	 				Transfer_Antagonist_Str = "Superposition", // or "Multiplied"
	 				Transfer_Steering_Str = "Superposition"; // or "Multiplied"

	Choice		Formula,
				Steering,
				Transfer_Antagonist,
				Transfer_Steering;			
	
			
	public int 	Init_RC1 	= 40, // initialisation and reset values
				Init_RC2 	= 41,
				Init_RC3 	= 42,
				Init_RC4 	= 43,
				Init_RC5 	= 44,
				Init_RC6 	= 45,
				Init_RC7 	= 40, 
				Init_RC8 	= 41,
				Init_RC9 	= 42,
				Init_RC10 	= 43,
				Init_RC11 	= 44,
				Init_RC12 	= 45,
				Init_Amp1	= 50,
				Init_Amp2	= 51,
				Init_Amp3	= 52,
				Init_Amp4	= 53,
				Init_Amp5	= 54,
				Init_Amp6	= 55,
				Init_Time1	= - 400,
				Init_Time2	= - 400,
				Init_Time3	= - 400,
				Init_Time4	= - 400,
				Init_Time5	= - 400,
				Init_Time6	= - 400;
				
				
	public int  RC1 	=Init_RC1, // int tied to scrollbars
				RC2 	=Init_RC2,
				RC3		=Init_RC3,
				RC4 	=Init_RC4,
				RC5		=Init_RC5,
				RC6 	=Init_RC6,
				RC7 	=Init_RC7, 
				RC8 	=Init_RC8,
				RC9		=Init_RC9,
				RC10 	=Init_RC10,
				RC11	=Init_RC11,
				RC12 	=Init_RC12,
				Amp1 	=Init_Amp1,
				Amp2 	=Init_Amp2,
				Amp3 	=Init_Amp3,
				Amp4 	=Init_Amp4,
				Amp5 	=Init_Amp5,
				Amp6 	=Init_Amp6,
				Time1 	=Init_Time1,
				Time2 	=Init_Time2,
				Time3	=Init_Time3,
				Time4 	=Init_Time4,
				Time5 	=Init_Time5,
				Time6 	=Init_Time6;
				
	
	TextField	
				RC1_TEXT, // textfields for values of scrollbars (exept time differences)
				RC2_TEXT,
				RC3_TEXT,
				RC4_TEXT,
				RC5_TEXT,
				RC6_TEXT,
				RC7_TEXT, 
				RC8_TEXT,
				RC9_TEXT,
				RC10_TEXT,
				RC11_TEXT,
				RC12_TEXT,
				Amp1_TEXT,
				Amp2_TEXT,
				Amp3_TEXT,
				Amp4_TEXT,
				Amp5_TEXT,
				Amp6_TEXT,
				Time1_TEXT,
				Time2_TEXT,
				Time3_TEXT,
				Time4_TEXT,
				Time5_TEXT,
				Time6_TEXT;
				
				
	Button 		Reset_Button, // buttons to go back to original manual/train settings
				Go_Button; // button to make robot drive in manual setting
				
	Checkbox	Invert1, // choices to invert muscle graphs
				Invert2,
				Invert3,
				Invert4,
				Invert5,
				Invert6;
								
	Scrollbar 	
				RC1_SCROLL, // scrollbars to manually select RC values for generalised muscle
				RC2_SCROLL,
				RC3_SCROLL,
				RC4_SCROLL,
				RC5_SCROLL,
				RC6_SCROLL,
				RC7_SCROLL, 
				RC8_SCROLL,
				RC9_SCROLL,
				RC10_SCROLL,
				RC11_SCROLL,
				RC12_SCROLL,
				Amp1_SCROLL, // "" for Amplification values
				Amp2_SCROLL,
				Amp3_SCROLL,
				Amp4_SCROLL,
				Amp5_SCROLL,
				Amp6_SCROLL,
				Time1_SCROLL, // "" for time delays
				Time2_SCROLL,
				Time3_SCROLL,
				Time4_SCROLL,
				Time5_SCROLL,
				Time6_SCROLL;
								
	PathCanvas 	Path_C;
	SpeedCanvas Up_C, 
				Down_C;
				
	Formulas	Calculate;
			
//******************************************************************	
//**** Initialisation of applet, layout and initial values      ****
//******************************************************************	

	public void init()
	{
		Reset_Button = new Button("RESET");
		Go_Button	= new Button("GO"); 
		
		// Scrollbar : orietation, initial value, visible amount, min, max
		RC1_SCROLL 	= new Scrollbar(Scrollbar.HORIZONTAL	, Init_RC1 , 1, 1, RC_Max);
		RC2_SCROLL 	= new Scrollbar(Scrollbar.HORIZONTAL	, Init_RC2 , 1, 1, RC_Max);
		RC3_SCROLL 	= new Scrollbar(Scrollbar.HORIZONTAL	, Init_RC3 , 1, 1, RC_Max);
		RC4_SCROLL 	= new Scrollbar(Scrollbar.HORIZONTAL	, Init_RC4 , 1, 1, RC_Max);
		RC5_SCROLL 	= new Scrollbar(Scrollbar.HORIZONTAL	, Init_RC5 , 1, 1, RC_Max);
		RC6_SCROLL 	= new Scrollbar(Scrollbar.HORIZONTAL	, Init_RC6 , 1, 1, RC_Max);
		RC7_SCROLL 	= new Scrollbar(Scrollbar.HORIZONTAL	, Init_RC7 , 1, 1, RC_Max);
		RC8_SCROLL 	= new Scrollbar(Scrollbar.HORIZONTAL	, Init_RC8 , 1, 1, RC_Max);
		RC9_SCROLL 	= new Scrollbar(Scrollbar.HORIZONTAL	, Init_RC9 , 1, 1, RC_Max);
		RC10_SCROLL = new Scrollbar(Scrollbar.HORIZONTAL	, Init_RC10 , 1, 1, RC_Max);
		RC11_SCROLL = new Scrollbar(Scrollbar.HORIZONTAL	, Init_RC11 , 1, 1, RC_Max);
		RC12_SCROLL = new Scrollbar(Scrollbar.HORIZONTAL	, Init_RC12 , 1, 1, RC_Max);
		Amp1_SCROLL = new Scrollbar(Scrollbar.VERTICAL		, Init_Amp1, 0, 1, Amp_Max);
		Amp2_SCROLL = new Scrollbar(Scrollbar.VERTICAL		, Init_Amp2, 0, 1, Amp_Max);
		Amp3_SCROLL = new Scrollbar(Scrollbar.VERTICAL		, Init_Amp3, 0, 1, Amp_Max);
		Amp4_SCROLL = new Scrollbar(Scrollbar.VERTICAL		, Init_Amp4, 0, 1, Amp_Max);
		Amp5_SCROLL = new Scrollbar(Scrollbar.VERTICAL		, Init_Amp5, 0, 1, Amp_Max);
		Amp6_SCROLL = new Scrollbar(Scrollbar.VERTICAL		, Init_Amp6, 0, 1, Amp_Max);
		Time1_SCROLL = new Scrollbar(Scrollbar.HORIZONTAL		, Init_Time1, 1, -Time_Max , Time_Max );
		Time2_SCROLL = new Scrollbar(Scrollbar.HORIZONTAL		, Init_Time2, 1, -Time_Max , Time_Max );
		Time3_SCROLL = new Scrollbar(Scrollbar.HORIZONTAL		, Init_Time3, 1, -Time_Max , Time_Max );
		Time4_SCROLL = new Scrollbar(Scrollbar.HORIZONTAL		, Init_Time4, 1, -Time_Max , Time_Max );
		Time5_SCROLL = new Scrollbar(Scrollbar.HORIZONTAL		, Init_Time5, 1, -Time_Max , Time_Max );
		Time6_SCROLL = new Scrollbar(Scrollbar.HORIZONTAL		, Init_Time6, 1, -Time_Max , Time_Max );

		RC1_TEXT 	= new TextField(String.valueOf(RC1  ), 1);
		RC2_TEXT 	= new TextField(String.valueOf(RC2  ), 1);
		RC3_TEXT 	= new TextField(String.valueOf(RC3  ), 1);
		RC4_TEXT 	= new TextField(String.valueOf(RC4  ), 1);
		RC5_TEXT 	= new TextField(String.valueOf(RC5  ), 1);
		RC6_TEXT 	= new TextField(String.valueOf(RC6  ), 1);
		RC7_TEXT 	= new TextField(String.valueOf(RC7  ), 1);
		RC8_TEXT 	= new TextField(String.valueOf(RC8  ), 1);
		RC9_TEXT 	= new TextField(String.valueOf(RC9  ), 1);
		RC10_TEXT 	= new TextField(String.valueOf(RC10 ), 1);
		RC11_TEXT 	= new TextField(String.valueOf(RC11 ), 1);
		RC12_TEXT 	= new TextField(String.valueOf(RC12 ), 1);
		Amp1_TEXT 	= new TextField(String.valueOf(Amp1 ), 1);
		Amp2_TEXT 	= new TextField(String.valueOf(Amp2 ), 1);
		Amp3_TEXT 	= new TextField(String.valueOf(Amp3 ), 1);
		Amp4_TEXT 	= new TextField(String.valueOf(Amp4 ), 1);
		Amp5_TEXT 	= new TextField(String.valueOf(Amp5 ), 1);
		Amp6_TEXT 	= new TextField(String.valueOf(Amp6 ), 1);
		Time1_TEXT 	= new TextField(String.valueOf(Time1 + 400), 1);
		Time2_TEXT 	= new TextField(String.valueOf(Time2 + 400), 1);
		Time3_TEXT 	= new TextField(String.valueOf(Time3 + 400), 1);
		Time4_TEXT 	= new TextField(String.valueOf(Time4 + 400), 1);
		Time5_TEXT 	= new TextField(String.valueOf(Time5 + 400), 1);
		Time6_TEXT 	= new TextField(String.valueOf(Time6 + 400), 1);
		
		RC1_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		RC2_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		RC3_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		RC4_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		RC5_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		RC6_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		RC7_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		RC8_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		RC9_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		RC10_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		RC11_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		RC12_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		Amp1_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		Amp2_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		Amp3_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		Amp4_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		Amp5_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		Amp6_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		Time1_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		Time2_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		Time3_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		Time4_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		Time5_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
		Time6_TEXT.setFont(new Font("Plain", Font.BOLD, 10));
				
		// choice initialisations with right parameters
		Formula = new Choice();
			Formula.add("RcRcfilter");
			Formula.add("Sinus");
			Formula.add("Gauss");
		Formula.addItemListener(this);	// actionlistener choicebox	
		
		Steering = new Choice();
			Steering.add("Speed");
			Steering.add("Carthesian");
			Steering.add("Energy");
		Steering.addItemListener(this);	

		Transfer_Antagonist = new Choice();
			Transfer_Antagonist.add("Superposition");
			Transfer_Antagonist.add("Multiplied");
		Transfer_Antagonist.addItemListener(this);	

		Transfer_Steering = new Choice();
			Transfer_Steering.add("Superposition");
			Transfer_Steering.add("Multiplied");
		Transfer_Steering.addItemListener(this);	
		
		Invert1 = new Checkbox("Invert Yellow up", false);
		Invert2 = new Checkbox("Invert Red up", false);
		Invert3 = new Checkbox("Invert Green up", false);
		Invert4 = new Checkbox("Invert Yellow down", false);		
		Invert5 = new Checkbox("Invert Red down", false);		
		Invert6 = new Checkbox("Invert Green down", false);		
			
		Invert1.setBackground(Color.lightGray);		
		Invert2.setBackground(Color.lightGray);		
		Invert3.setBackground(Color.lightGray);		
		Invert4.setBackground(Color.lightGray);		
		Invert5.setBackground(Color.lightGray);		
		Invert6.setBackground(Color.lightGray);		
		
		// actionlisteners buttons		
 		Reset_Button.addActionListener(this); 
		Go_Button.addActionListener(this);
		
		// listeners for scrollbars
		RC1_SCROLL.addAdjustmentListener(this);
		RC2_SCROLL.addAdjustmentListener(this);
		RC3_SCROLL.addAdjustmentListener(this);
		RC4_SCROLL.addAdjustmentListener(this);
		RC5_SCROLL.addAdjustmentListener(this);
		RC6_SCROLL.addAdjustmentListener(this);
		RC7_SCROLL.addAdjustmentListener(this);
		RC8_SCROLL.addAdjustmentListener(this);
		RC9_SCROLL.addAdjustmentListener(this);
		RC10_SCROLL.addAdjustmentListener(this);
		RC11_SCROLL.addAdjustmentListener(this);
		RC12_SCROLL.addAdjustmentListener(this);
		Amp1_SCROLL.addAdjustmentListener(this);
		Amp2_SCROLL.addAdjustmentListener(this);
		Amp3_SCROLL.addAdjustmentListener(this);
		Amp4_SCROLL.addAdjustmentListener(this);
		Amp5_SCROLL.addAdjustmentListener(this);
		Amp6_SCROLL.addAdjustmentListener(this);
		Time1_SCROLL.addAdjustmentListener(this);
		Time2_SCROLL.addAdjustmentListener(this);
		Time3_SCROLL.addAdjustmentListener(this);
		Time4_SCROLL.addAdjustmentListener(this);
		Time5_SCROLL.addAdjustmentListener(this);
		Time6_SCROLL.addAdjustmentListener(this);

		Invert1.addItemListener(this); 		
		Invert2.addItemListener(this); 		
		Invert3.addItemListener(this); 		
		Invert4.addItemListener(this); 		
		Invert5.addItemListener(this); 		
		Invert6.addItemListener(this); 		
				
		Up_C = 		new SpeedCanvas(Sample_Max, "Up", Formula_Str);
		Down_C = 	new SpeedCanvas(Sample_Max, "Down", Formula_Str);
		Path_C = 	new PathCanvas(Sample_Max);
		Calculate = new Formulas(Sample_Size, Sample_Max);
		
		setBackground(Color.black);
		setLayout(new GridLayout(1, 2)); // toplayergrid (y, x) : Muscles next to path 
			Panel PathPlot = new Panel(); // in here the path of the robot is drawn
			PathPlot.setLayout(new BorderLayout());  
				Panel Ch_Inv = new Panel();
				Ch_Inv.setLayout(new BorderLayout());
					Panel Choices = new Panel(); // here choices for choice and inversion boxes are added
					Choices.setLayout(new GridLayout(1, 4)); 
					Choices.add(Formula);
					Choices.add(Steering);
					Choices.add(Transfer_Antagonist);
					Choices.add(Transfer_Steering);
					Panel Inversions = new Panel();
					Inversions.setLayout(new GridLayout(2,3));
					Inversions.add(Invert1);
					Inversions.add(Invert2);
					Inversions.add(Invert3);
					Inversions.add(Invert4);
					Inversions.add(Invert5);
					Inversions.add(Invert6);
				Ch_Inv.add("North", Choices);
				Ch_Inv.add("Center", Inversions);
			PathPlot.add("North", Ch_Inv); // add the inverses and choices	
			PathPlot.add("Center", Path_C);	// add the graphics canvas
				Panel Buttons = new Panel(); // panel to add buttons
			   	Buttons.setLayout(new GridLayout(1,2)); // here the two buttons are added
			   		Buttons.add(Reset_Button);
			   		Buttons.add(Go_Button);
			PathPlot.add("South", Buttons);
		add(PathPlot);		
			Panel Graphs = new Panel();
			Graphs.setLayout(new GridLayout(2, 1)); // in here the two graphs are drawn
				Panel Graph_Up = new Panel();
				Graph_Up.setLayout(new BorderLayout());
				Graph_Up.add("Center", Up_C);
					Panel UP_SOUTH = new Panel();
					UP_SOUTH.setLayout(new GridLayout(3, 3));
						Panel Time1_GR = new Panel();
						Time1_GR.setLayout(new BorderLayout());
						Time1_GR.add("East", Time1_TEXT);
						Time1_GR.add("Center", Time1_SCROLL);
						Panel Time2_GR = new Panel();
						Time2_GR.setLayout(new BorderLayout());
						Time2_GR.add("East", Time2_TEXT);
						Time2_GR.add("Center", Time2_SCROLL);
						Panel Time3_GR = new Panel();
						Time3_GR.setLayout(new BorderLayout());
						Time3_GR.add("East", Time3_TEXT);
						Time3_GR.add("Center", Time3_SCROLL);
						Panel RC1_GR = new Panel();
						RC1_GR.setLayout(new BorderLayout());
						RC1_GR.add("East", RC1_TEXT);
						RC1_GR.add("Center", RC1_SCROLL);
						Panel RC2_GR = new Panel();
						RC2_GR.setLayout(new BorderLayout());
						RC2_GR.add("East", RC2_TEXT);
						RC2_GR.add("Center", RC2_SCROLL);
						Panel RC3_GR = new Panel();
						RC3_GR.setLayout(new BorderLayout());
						RC3_GR.add("East", RC3_TEXT);
						RC3_GR.add("Center", RC3_SCROLL);
						Panel RC4_GR = new Panel();
						RC4_GR.setLayout(new BorderLayout());
						RC4_GR.add("East", RC4_TEXT);
						RC4_GR.add("Center", RC4_SCROLL);
						Panel RC5_GR = new Panel();
						RC5_GR.setLayout(new BorderLayout());
						RC5_GR.add("East", RC5_TEXT);
						RC5_GR.add("Center", RC5_SCROLL);
						Panel RC6_GR = new Panel();
						RC6_GR.setLayout(new BorderLayout());
						RC6_GR.add("East", RC6_TEXT);
						RC6_GR.add("Center", RC6_SCROLL);
					UP_SOUTH.add(Time1_GR);	
					UP_SOUTH.add(RC1_GR);	
					UP_SOUTH.add(RC2_GR);	
					UP_SOUTH.add(Time2_GR);	
					UP_SOUTH.add(RC3_GR);	
					UP_SOUTH.add(RC4_GR);	
					UP_SOUTH.add(Time3_GR);	
					UP_SOUTH.add(RC5_GR);	
					UP_SOUTH.add(RC6_GR);	
				Graph_Up.add("South", UP_SOUTH);
					Panel UP_EAST = new Panel();
					UP_EAST.setLayout(new BorderLayout());
						Panel Amp1_GR = new Panel();
						Amp1_GR.setLayout(new GridLayout(2, 1));
						Amp1_GR.add(Amp1_TEXT);
						Amp1_GR.add(Amp2_TEXT);
						Panel Amp2_GR = new Panel();
						Amp2_GR.setLayout(new GridLayout(1, 2));
						Amp2_GR.add(Amp1_SCROLL);
						Amp2_GR.add(Amp2_SCROLL);
					UP_EAST.add("North", Amp1_GR);
					UP_EAST.add("Center", Amp2_GR);
				Graph_Up.add("West", UP_EAST);
					Panel Amp3_GR = new Panel();
					Amp3_GR.setLayout(new BorderLayout());
					Amp3_GR.add("North", Amp3_TEXT);
					Amp3_GR.add("Center", Amp3_SCROLL);
				Graph_Up.add("East", Amp3_GR);
			Graphs.add(Graph_Up);
				Panel Graph_Down = new Panel();
				Graph_Down.setLayout(new BorderLayout());
				Graph_Down.add("Center", Down_C);
					Panel DOWN_SOUTH = new Panel();
					DOWN_SOUTH.setLayout(new GridLayout(3, 3));
						Panel Time4_GR = new Panel();
						Time4_GR.setLayout(new BorderLayout());
						Time4_GR.add("East", Time4_TEXT);
						Time4_GR.add("Center", Time4_SCROLL);
						Panel Time5_GR = new Panel();
						Time5_GR.setLayout(new BorderLayout());
						Time5_GR.add("East", Time5_TEXT);
						Time5_GR.add("Center", Time5_SCROLL);
						Panel Time6_GR = new Panel();
						Time6_GR.setLayout(new BorderLayout());
						Time6_GR.add("East", Time6_TEXT);
						Time6_GR.add("Center", Time6_SCROLL);
						Panel RC7_GR = new Panel();
						RC7_GR.setLayout(new BorderLayout());
						RC7_GR.add("East", RC7_TEXT);
						RC7_GR.add("Center", RC7_SCROLL);
						Panel RC8_GR = new Panel();
						RC8_GR.setLayout(new BorderLayout());
						RC8_GR.add("East", RC8_TEXT);
						RC8_GR.add("Center", RC8_SCROLL);
						Panel RC9_GR = new Panel();
						RC9_GR.setLayout(new BorderLayout());
						RC9_GR.add("East", RC9_TEXT);
						RC9_GR.add("Center", RC9_SCROLL);
						Panel RC10_GR = new Panel();
						RC10_GR.setLayout(new BorderLayout());
						RC10_GR.add("East", RC10_TEXT);
						RC10_GR.add("Center", RC10_SCROLL);
						Panel RC11_GR = new Panel();
						RC11_GR.setLayout(new BorderLayout());
						RC11_GR.add("East", RC11_TEXT);
						RC11_GR.add("Center", RC11_SCROLL);
						Panel RC12_GR = new Panel();
						RC12_GR.setLayout(new BorderLayout());
						RC12_GR.add("East", RC12_TEXT);
						RC12_GR.add("Center", RC12_SCROLL);
					DOWN_SOUTH.add(Time4_GR);	
					DOWN_SOUTH.add(RC7_GR);	
					DOWN_SOUTH.add(RC8_GR);	
					DOWN_SOUTH.add(Time5_GR);	
					DOWN_SOUTH.add(RC9_GR);	
					DOWN_SOUTH.add(RC10_GR);	
					DOWN_SOUTH.add(Time6_GR);	
					DOWN_SOUTH.add(RC11_GR);	
					DOWN_SOUTH.add(RC12_GR);	
				Graph_Down.add("South", DOWN_SOUTH);
					Panel DOWN_EAST = new Panel();
					DOWN_EAST.setLayout(new BorderLayout());
						Panel Amp4_GR = new Panel();
						Amp4_GR.setLayout(new GridLayout(2, 1));
						Amp4_GR.add(Amp4_TEXT);
						Amp4_GR.add(Amp5_TEXT);
						Panel Amp5_GR = new Panel();
						Amp5_GR.setLayout(new GridLayout(1, 2));
						Amp5_GR.add(Amp4_SCROLL);
						Amp5_GR.add(Amp5_SCROLL);
					DOWN_EAST.add("North", Amp4_GR);
					DOWN_EAST.add("Center",Amp5_GR);
				Graph_Down.add("West", DOWN_EAST);
					Panel Amp6_GR = new Panel();
					Amp6_GR.setLayout(new BorderLayout());
					Amp6_GR.add("North", Amp6_TEXT);
					Amp6_GR.add("Center", Amp6_SCROLL);
				Graph_Down.add("East", Amp6_GR);
			Graphs.add(Graph_Down);
		add(Graphs);		

		FillBuffers(Up_C);
		FillBuffers(Down_C);
//		NewPath();
		UpdateDrawing();
				
	}
	
//******************************************************************	
//**** Listeners for actions in applet                          ****
//******************************************************************	

	public void itemStateChanged(ItemEvent evt) 
	{ // listener for choice box formula chooses what formula to be used for muscle
		if (evt.getItemSelectable()  == Formula)
		{
			if(evt.getItem().equals("RcRcfilter")) 	
				{
					Formula_Str = "RcRcfilter"; 
					Up_C.Mode_Str = "RcRcfilter";
					Down_C.Mode_Str = "RcRcfilter";
				}
			else if(evt.getItem().equals("Sinus")) 
			{
				Formula_Str = "Sinus";
				Up_C.Mode_Str = "Sinus";
				Down_C.Mode_Str = "Sinus";
			}
			else if(evt.getItem().equals("Gauss")) 	
			{
				Formula_Str = "Gauss";
				Up_C.Mode_Str = "Gauss";
				Down_C.Mode_Str = "Gauss";
			}
				
		} 
		else if (evt.getItemSelectable()  == Steering)
		{
			if(evt.getItem().equals("Carthesian")) 	{Steering_Str = "Carthesian";}
			if(evt.getItem().equals("Speed")) 		{Steering_Str = "Speed";}			
			if(evt.getItem().equals("Energy")) 		{Steering_Str = "Energy";}			
		}
		else if (evt.getItemSelectable()  == Transfer_Antagonist)
		{
			if(evt.getItem().equals("Superposition")) 	{Transfer_Antagonist_Str = "Superposition";}			
			if(evt.getItem().equals("Multiplied")) 		{Transfer_Antagonist_Str = "Multiplied";}			
		}
		else if (evt.getItemSelectable()  == Transfer_Steering)
		{
			if(evt.getItem().equals("Superposition")) 	{Transfer_Steering_Str = "Superposition";}			
			if(evt.getItem().equals("Multiplied")) 		{Transfer_Steering_Str = "Multiplied";}			
		}
		else if(evt.getItemSelectable()  == Invert1)
		{
		} 
		else if(evt.getItemSelectable() == Invert2)
		{
		} 
		else if(evt.getItemSelectable() == Invert3)
		{
		} 
		else if(evt.getItemSelectable() == Invert4)
		{
		}
		else if(evt.getItemSelectable() == Invert5)
		{
		}
		else if(evt.getItemSelectable() == Invert6)
		{
		}
		FillBuffers(Up_C);
		FillBuffers(Down_C);
		NewPath();
		UpdateDrawing();
  	}

  	public void UpdateDrawing()
  	{
		Path_C.repaint();
	  	Up_C.repaint();
		Down_C.repaint();
  	}

  	public void NewPath() // caluculates path for the changes made
  	{
	  	
	}
		  	
 	public void FillBuffers(SpeedCanvas Buf) // gives away which canvas is filled
  	{
	  	double Temp1[] = new double[Sample_Size];
	  	double Temp2[] = new double[Sample_Size];
	  	double Temp3[] = new double[Sample_Size];
	  	
	  	if(Buf.Name_Str == "Up")
	  	{
	  		for(int i = 0; i < Sample_Size; i++)
		  	{
		  		if(!Invert1.getState()) 
		  		{
		  			Temp1[i] = Calculate.Next_Steer(Formula_Str, RC1, RC2, Amp1, i);
	  			} else Temp1[Sample_Size - i - 1] = Calculate.Next_Steer(Formula_Str, RC1, RC2, Amp1, i);
	  			
		  		if(!Invert2.getState()) 
		  		{
			  		Temp2[i] = Calculate.Next_Steer(Formula_Str, RC3, RC4, Amp2, i);
				} else Temp2[Sample_Size - i - 1] = Calculate.Next_Steer(Formula_Str, RC3, RC4, Amp2, i);
				
			  	if(!Invert3.getState()) 
		  		{
		  			Temp3[i] = Calculate.Next_Steer(Formula_Str, RC5, RC6, Amp3, i);
	  			} else Temp3[Sample_Size - i - 1] = Calculate.Next_Steer(Formula_Str, RC5, RC6, Amp3, i);
	  		} 
	  		
		  		
	  	} else // Buf.Name_Str name is apparently "Down"
	  	{
	  		for(int i = 0; i < Sample_Size; i++)
		  	{
		  		if(!Invert4.getState()) 
		  		{
			  		Temp1[i] = Calculate.Next_Steer(Formula_Str, RC7,  RC8  , Amp4, i);
		  		} else Temp1[Sample_Size - i - 1] = Calculate.Next_Steer(Formula_Str, RC7,  RC8  , Amp4, i);
		  		
		  		if(!Invert5.getState()) 
		  		{
			  		Temp2[i] = Calculate.Next_Steer(Formula_Str, RC9,  RC10 , Amp5, i);
		  		} else Temp2[Sample_Size - i - 1] = Calculate.Next_Steer(Formula_Str, RC9,  RC10 , Amp5, i);
		  		
		  		if(!Invert6.getState()) 
		  		{
			  		Temp3[i] = Calculate.Next_Steer(Formula_Str, RC11, RC12 , Amp6, i);
		  		} else Temp3[Sample_Size - i - 1] = Calculate.Next_Steer(Formula_Str, RC11, RC12 , Amp6, i);
	  		} 
	  				  		
  		}
	  	
	  	// we now have three graph buffers, agonist, antagonist and steering influence
	  	// these can now be filled in in the right canvas at the right place (time displacement)

	  	Buf.Up_Buffer = new double[Sample_Max]; // initialize all buffers at zero
	  	Buf.Down_Buffer = new double[Sample_Max];
	  	Buf.Steer_Buffer = new double[Sample_Max];
	  		  	
	  	if(Buf.Name_Str == "Up")
	  	{
		  	for(int i = 0; i < Sample_Size; i++)
		  	{
			  	if((((Sample_Max / 2) + Time1 + i) > 0) && (((Sample_Max / 2) + Time1 + i) < Sample_Max)) 
			  	{ 	  	
						Buf.Up_Buffer[(Sample_Max / 2) + Time1 + i] = Temp1[i]; 
				} else Buf.Up_Buffer[i] = 0;
			  	if((((Sample_Max / 2) + Time2 + i) > 0) && (((Sample_Max / 2) + Time2 + i) < Sample_Max)) 
			  	{
				  		Buf.Down_Buffer[(Sample_Max / 2) + Time2 + i] = Temp2[i];
			  	} else Buf.Down_Buffer[i] = 0;
			  	if((((Sample_Max / 2) + Time3 + i) > 0) && (((Sample_Max / 2) + Time3 + i) < Sample_Max)) 
			  	{
		  				Buf.Steer_Buffer[(Sample_Max / 2) + Time3 + i] = Temp3[i];
	  			} else Buf.Steer_Buffer[i] = 0;
	  		}
	  	} else // Buf.Name_Str name is apparently "Down"
	  	{
		  	for(int i = 0; i < Sample_Size; i++)
		  	{
			  	if((((Sample_Max / 2) + Time4 + i) > 0) && (((Sample_Max / 2) + Time4 + i) < Sample_Max))  
			  	{
			  			Buf.Up_Buffer[(Sample_Max / 2) + Time4 + i] = Temp1[i]; 
		  		} else Buf.Up_Buffer[i] = 0;
			  	if((((Sample_Max / 2) + Time5 + i) > 0) && (((Sample_Max / 2) + Time5 + i) < Sample_Max))  
				{
				  		Buf.Down_Buffer[(Sample_Max / 2) + Time5 + i] = Temp2[i];
				} else Buf.Down_Buffer[i] = 0;
			  	if((((Sample_Max / 2) + Time6 + i) > 0) && (((Sample_Max / 2) + Time6 + i) < Sample_Max))  
			  	{
		  				Buf.Steer_Buffer[(Sample_Max / 2) + Time6 + i] = Temp3[i];
	  			} else Buf.Steer_Buffer[i] = 0;
	  		} 
	  	}
	  	
	 
 		if(Transfer_Antagonist_Str == "Superposition")
  		{
	  		if(Transfer_Steering_Str == "Superposition")
		  	{
			  	for(int i = 0; i < Sample_Max;i ++)
	  			{
			  		Buf.Up_Buffer[i] = Buf.Up_Buffer[i] - Buf.Down_Buffer[i] + Buf.Steer_Buffer[i];
	  			}
  			} else // Transfer_Steering_Str appears to be "Multiplied"
  			{ // i take it the antagonist goes first in this relation
			  	for(int i = 0; i < Sample_Max;i ++)
	  			{
			  		Buf.Up_Buffer[i] = (Buf.Up_Buffer[i] - Buf.Down_Buffer[i]) * (1 - (double)Buf.Steer_Buffer[i]/(double)Amp_Max);
	  			}	  			
  			} 			
  		} else // ergo Transfer_Antagonist_Str = " Multiplication"
  		{
	  		if(Transfer_Steering_Str == "Multiplied")
		  	{
		  		for(int i = 0; i < Sample_Max;i ++)
	  			{
			  		Buf.Up_Buffer[i] = (Buf.Up_Buffer[i] * (1 - ((double)Buf.Down_Buffer[i]/(double)Amp_Max))) * (1 - (double)Buf.Steer_Buffer[i]/(double)Amp_Max);
	  			}
  			} else // apparently Transfer_Steering_Str is "Superposition"
  			{
		  		for(int i = 0; i < Sample_Max;i ++)
	  			{
			  		Buf.Up_Buffer[i] = (Buf.Up_Buffer[i] * (1 - ((double)Buf.Down_Buffer[i]/(double)Amp_Max))) + Buf.Steer_Buffer[i];
	  			}
  			}
  		}
  		
  		// now we can use both filled buffers for the upper and lower canvas to make a path
  		
  		Path_C.X_Buffer 	= new double[Sample_Max];
  		Path_C.Y_Buffer 	= new double[Sample_Max];
  		Path_C.Phi_Buffer 	= new double[Sample_Max];
  		
  		for(int i = 1; i < Sample_Max; i++)
  		{
	  		double result[] = new double[3];
	  		double 	X = Path_C.X_Buffer[i - 1], 
	  				Y = Path_C.Y_Buffer[i - 1],
	  				Phi = Path_C.Phi_Buffer[i - 1];	  		
	  		
	  		result = Calculate.Next_Path(Steering_Str, X, Y, Up_C.Up_Buffer[i], Down_C.Up_Buffer[i], Phi);
	  		
	  		Path_C.X_Buffer[i] = result[0] ;
	  		Path_C.Y_Buffer[i] = result[1] ;
	  		Path_C.Phi_Buffer[i] = result[2];
  		}
	}
  		
    public void adjustmentValueChanged(AdjustmentEvent evt) 
    { 	
		if (evt.getSource() == RC1_SCROLL) 
		{
			RC1 = RC1_SCROLL.getValue();
			if(RC1 == RC2) RC1++;
			RC1_TEXT.setText(String.valueOf(RC1));
			FillBuffers(Up_C);
		} 
		else if (evt.getSource() == RC2_SCROLL) 
		{
			RC2 = RC2_SCROLL.getValue();
			if(RC1 == RC2) RC2++;
			RC2_TEXT.setText(String.valueOf(RC2));
			FillBuffers(Up_C);
		}	    
		else if (evt.getSource() == RC3_SCROLL) 
		{
			RC3 = RC3_SCROLL.getValue();
			if(RC3 == RC4) RC3++;
			RC3_TEXT.setText(String.valueOf(RC3));
			FillBuffers(Up_C);
		}	    
		else if (evt.getSource() == RC4_SCROLL) 
		{
			RC4 = RC4_SCROLL.getValue();
			if(RC3 == RC4) RC4++;
			RC4_TEXT.setText(String.valueOf(RC4));
			FillBuffers(Up_C);
		}	    
		else if (evt.getSource() == RC5_SCROLL) 
		{
			RC5 = RC5_SCROLL.getValue();
			if(RC5 == RC6) RC5++;
			RC5_TEXT.setText(String.valueOf(RC5));
			FillBuffers(Up_C);
		}	    
		else if (evt.getSource() == RC6_SCROLL) 
		{
			RC6 = RC6_SCROLL.getValue();
			if(RC5 == RC6) RC6++;
			RC6_TEXT.setText(String.valueOf(RC6));
			FillBuffers(Up_C);
		}
		if (evt.getSource() == RC7_SCROLL) 
		{
			RC7 = RC7_SCROLL.getValue();
			if(RC7 == RC8) RC7++;
			RC7_TEXT.setText(String.valueOf(RC7));
			FillBuffers(Down_C);
		} 
		else if (evt.getSource() == RC8_SCROLL) 
		{
			RC8 = RC8_SCROLL.getValue();
			if(RC7 == RC8) RC8++;
			RC8_TEXT.setText(String.valueOf(RC8));
			FillBuffers(Down_C);
		}	    
		else if (evt.getSource() == RC9_SCROLL) 
		{
			RC9 = RC9_SCROLL.getValue();
			if(RC9 == RC10) RC9++;
			RC9_TEXT.setText(String.valueOf(RC9));
			FillBuffers(Down_C);
		}	    
		else if (evt.getSource() == RC10_SCROLL) 
		{
			RC10 = RC10_SCROLL.getValue();
			if(RC9 == RC10) RC10++;
			RC10_TEXT.setText(String.valueOf(RC10));
			FillBuffers(Down_C);
		}	    
		else if (evt.getSource() == RC11_SCROLL) 
		{
			RC11 = RC11_SCROLL.getValue();
			if(RC11 == RC12) RC11++;
			RC11_TEXT.setText(String.valueOf(RC11));
			FillBuffers(Down_C);
		}	    
		else if (evt.getSource() == RC12_SCROLL) 
		{
			RC12 = RC12_SCROLL.getValue();
			if(RC11 == RC12) RC12++;
			RC12_TEXT.setText(String.valueOf(RC12));
			FillBuffers(Down_C);
		}
			    
		else if (evt.getSource() == Amp1_SCROLL) 
		{
			Amp1 = Amp1_SCROLL.getValue();
			Amp1_TEXT.setText(String.valueOf(Amp1));
			FillBuffers(Up_C);
		}	    
		else if (evt.getSource() == Amp2_SCROLL) 
		{
			Amp2 = Amp2_SCROLL.getValue();
			Amp2_TEXT.setText(String.valueOf(Amp2));
			FillBuffers(Up_C);
		}	    
		else if (evt.getSource() == Amp3_SCROLL) 
		{
			Amp3 = Amp3_SCROLL.getValue();
			Amp3_TEXT.setText(String.valueOf(Amp3));
			FillBuffers(Up_C);
		}	    
		else if (evt.getSource() == Amp4_SCROLL) 
		{
			Amp4 = Amp4_SCROLL.getValue();
			Amp4_TEXT.setText(String.valueOf(Amp4));
			FillBuffers(Down_C);
		}	    
		else if (evt.getSource() == Amp5_SCROLL) 
		{
			Amp5 = Amp5_SCROLL.getValue();
			Amp5_TEXT.setText(String.valueOf(Amp5));
			FillBuffers(Down_C);
		}	    
		else if (evt.getSource() == Amp6_SCROLL) 
		{
			Amp6 = Amp6_SCROLL.getValue();
			Amp6_TEXT.setText(String.valueOf(Amp6));
			FillBuffers(Down_C);
		}	    
		
		else if (evt.getSource() == Time1_SCROLL) 
		{
			Time1 = Time1_SCROLL.getValue();
			Time1_TEXT.setText(String.valueOf(Time1 + 400));
			FillBuffers(Up_C);
		}	    
		else if (evt.getSource() == Time2_SCROLL) 
		{
			Time2 = Time2_SCROLL.getValue();
			Time2_TEXT.setText(String.valueOf(Time2 + 400));
			FillBuffers(Up_C);
		}	    
		else if (evt.getSource() == Time3_SCROLL) 
		{
			Time3 = Time3_SCROLL.getValue();
			Time3_TEXT.setText(String.valueOf(Time3 + 400));
			FillBuffers(Up_C);
		}	    
		else if (evt.getSource() == Time4_SCROLL) 
		{
			Time4 = Time4_SCROLL.getValue();
			Time4_TEXT.setText(String.valueOf(Time4 + 400));
			FillBuffers(Down_C);
		}	    
		else if (evt.getSource() == Time5_SCROLL) 
		{
			Time5 = Time5_SCROLL.getValue();
			Time5_TEXT.setText(String.valueOf(Time5 + 400));
			FillBuffers(Down_C);
		}	    
		else if (evt.getSource() == Time6_SCROLL) 
		{
			Time6 = Time6_SCROLL.getValue();
			Time6_TEXT.setText(String.valueOf(Time6 + 400));
			FillBuffers(Down_C);
		}	    
		NewPath();
		UpdateDrawing();
	}
	
	ActionListener taskPerformer = new ActionListener()
	{
		public void actionPerformed(ActionEvent evt) 
		{
			Path_C.Ticker++;
			Path_C.Ticker++;
			if(Path_C.Animate) Path_C.repaint();
			
			// hier moet nog even worden gekeken of een einde aan kan worden gebreid

			if(Path_C.Ticker > (Sample_Max - 1))
			{
				Path_C.Animate = false;
				Path_C.Ticker = 0;
				timer.stop();
			} 
		}
	};
		 		
	private int SearchStart()
	{
		int result = 0;
		
		while((Path_C.X_Buffer[result] == 0) && (Path_C.Y_Buffer[result] == 0) && (result < Sample_Max - 1)) result++;
		
		return result;
	}
	
    public void actionPerformed(ActionEvent evt)  
    {
	    Object source = evt.getSource();
	    
		if (source == Reset_Button) // reset is pushed so reset to initvaribles
		{
			RC1 	=Init_RC1; // setting initialisation variables for scrollbars
			RC2 	=Init_RC2;
			RC3		=Init_RC3;
			RC4 	=Init_RC4;
			RC5		=Init_RC5;
			RC6 	=Init_RC6;
			RC7 	=Init_RC7; 
			RC8 	=Init_RC8;
			RC9		=Init_RC9;
			RC10 	=Init_RC10;
			RC11	=Init_RC11;
			RC12 	=Init_RC12;
			Amp1 	=Init_Amp1;
			Amp2 	=Init_Amp2;
			Amp3 	=Init_Amp3;
			Amp4 	=Init_Amp4;
			Amp5 	=Init_Amp5;
			Amp6 	=Init_Amp6;
			Time1 	=Init_Time1;
			Time2 	=Init_Time2;
			Time3	=Init_Time3;
			Time4 	=Init_Time4;
			Time5 	=Init_Time5;
			Time6 	=Init_Time6;
									
			RC1_SCROLL.setValue(RC1); // resetting of scrollbarsliders
			RC2_SCROLL.setValue(RC2);
			RC3_SCROLL.setValue(RC3);
			RC4_SCROLL.setValue(RC4);
			RC5_SCROLL.setValue(RC5);
			RC6_SCROLL.setValue(RC6);
			RC7_SCROLL.setValue(RC7); // resetting of scrollbarsliders
			RC8_SCROLL.setValue(RC8);
			RC9_SCROLL.setValue(RC9);
			RC10_SCROLL.setValue(RC10);
			RC11_SCROLL.setValue(RC11);
			RC12_SCROLL.setValue(RC12);
			Amp1_SCROLL.setValue(Amp1);
			Amp2_SCROLL.setValue(Amp2);
			Amp3_SCROLL.setValue(Amp3);
			Amp4_SCROLL.setValue(Amp4);
			Amp5_SCROLL.setValue(Amp5);
			Amp6_SCROLL.setValue(Amp6);
			Time1_SCROLL.setValue(Time1);
			Time2_SCROLL.setValue(Time2);
			Time3_SCROLL.setValue(Time3);
			Time4_SCROLL.setValue(Time4);
			Time5_SCROLL.setValue(Time5);
			Time6_SCROLL.setValue(Time6);
			
			RC1_TEXT.setText(String.valueOf(RC1)); // resetting of value indicators
			RC2_TEXT.setText(String.valueOf(RC2));
			RC3_TEXT.setText(String.valueOf(RC3));
			RC4_TEXT.setText(String.valueOf(RC4));
			RC5_TEXT.setText(String.valueOf(RC5));
			RC6_TEXT.setText(String.valueOf(RC6));
			RC7_TEXT.setText(String.valueOf(RC7)); 
			RC8_TEXT.setText(String.valueOf(RC8));
			RC9_TEXT.setText(String.valueOf(RC9));
			RC10_TEXT.setText(String.valueOf(RC10));
			RC11_TEXT.setText(String.valueOf(RC11));
			RC12_TEXT.setText(String.valueOf(RC12));

			Amp1_TEXT.setText(String.valueOf(Amp1));
			Amp2_TEXT.setText(String.valueOf(Amp2));
			Amp3_TEXT.setText(String.valueOf(Amp3));
			Amp4_TEXT.setText(String.valueOf(Amp4));
			Amp5_TEXT.setText(String.valueOf(Amp5));
			Amp6_TEXT.setText(String.valueOf(Amp6));

			Time1_TEXT.setText(String.valueOf(Time1));
			Time2_TEXT.setText(String.valueOf(Time2));
			Time3_TEXT.setText(String.valueOf(Time3));
			Time4_TEXT.setText(String.valueOf(Time4));
			Time5_TEXT.setText(String.valueOf(Time5));
			Time6_TEXT.setText(String.valueOf(Time6));
						
			Formula_Str = "RcRcfilter"; // resetting of choices .... not complete !!!
	 		Steering_Str = "Speed";
	 		Transfer_Antagonist_Str = "Superposition";
	 		Transfer_Steering_Str = "RcRcfilter"; 
			
			FillBuffers(Up_C);
			FillBuffers(Down_C);
			NewPath();
			UpdateDrawing();														
		} else
		if ((source == Go_Button) && (!Path_C.Animate)) 
		{
			int DELAY = 10; // delay 20 ms before next circle
			timer = new Timer(DELAY, taskPerformer);
			timer.start();
			Path_C.Ticker = SearchStart(); 
			Path_C.Animate = true; // setting the Animation loop active
		}
	}    					
}