//******************************************************************		
//*** 															 ***
//*** Author :  Alex Brussee									 ***
//*** 			xxlex@hotmail.com					9/07/2003 	 ***
//*** 															 ***
//******************************************************************		
//***                                                            ***
//*** "PathCanvas.java" draws the path of the robot using        ***
//*** path for x,y,phi positions at [t]                          ***
//***                                                            ***
//***                                                            ***
//******************************************************************	

import java.awt.*;
import java.applet.Applet;

public class PathCanvas extends Canvas
{
	private Image     offScreenImage;
	private Dimension offScreenSize;
	private Graphics  offScreenGraphics;
	
	
	public PathCanvas(int Sample_Max)
	{
		Samples = Sample_Max;
		X_Buffer = new double[Samples];
		Y_Buffer = new double[Samples];
		Phi_Buffer = new double[Samples];		
		Animate = false;
		Ticker = 0;
	}
	
	public final synchronized void update (Graphics g) 
	{
		Dimension d = this.getSize();
  		if((this.offScreenImage == null) || (d.width != this.offScreenSize.width) ||  (d.height != this.offScreenSize.height)) 
   		{
	    	this.offScreenImage = this.createImage(d.width, d.height);
	    	this.offScreenSize = d;
	    	this.offScreenGraphics = this.offScreenImage.getGraphics();
  		}
  		
  		this.offScreenGraphics.clearRect(0, 0, d.width, d.height);
  		this.paint(this.offScreenGraphics);
  		g.drawImage(this.offScreenImage, 0, 0, null);
  	}

// Until now all thats done is take away the flicker, now the real drawing starts. 	
  
	private double 	scale = 100;
	private int 	Samples, radius = 10, radius2 = 20; // radius2 is double size raiud to avoid calculations
	private int 	w, h;
		
	public double 	X_Buffer[],
					Y_Buffer[],
	 				Phi_Buffer[]; 
	 				
	public boolean 	Animate;
	public int 		Ticker;
		
	public void paint(Graphics g) 
	{
		h = getSize().height; // get x/y sizes of canvas
		w = getSize().width;
		
		if(!Animate)
		{
			g.setColor(Color.white); // set line drawing color (not black : color of background)
			for(int t = 1; t < (Samples - 1); t++)
			{
				g.drawLine( (int)(w/6 + X_Buffer[t]/scale), (int)(h/2 + (Y_Buffer[t]/scale)), (int)((w/6 + X_Buffer[t - 1]/scale)),(int)(h/2 + (Y_Buffer[t - 1]/scale)));			
			}
		} else
		{								
			g.setColor(new Color(180, 180, 180));
			for(int t = 1; t < (Samples - 1); t++)
			{
				g.drawLine( (int)(w/6 + X_Buffer[t]/scale), (int)(h/2 + (Y_Buffer[t]/scale)), (int)((w/6 + X_Buffer[t - 1]/scale)),(int)(h/2 + (Y_Buffer[t - 1]/scale)));			
			}
			
			g.setColor(Color.green); // set line drawing color (not black : color of background)
			g.drawLine( (int)(w/6 + X_Buffer[Ticker]/scale ), (int)(h/2 + (Y_Buffer[Ticker]/scale)), 
						(int)(w/6 + X_Buffer[Ticker]/scale + 15 * Math.cos(Phi_Buffer[Ticker])),
						(int)(h/2 + Y_Buffer[Ticker]/scale + 15 * Math.sin(Phi_Buffer[Ticker])));
			g.fillOval( (int)(w/6 + X_Buffer[Ticker]/scale - radius), (int)(h/2 + (Y_Buffer[Ticker]/scale) - radius), radius2, radius2);			
			g.drawString(Integer.toString(Ticker), 10, 20);
			
															
		}
	} 

	
}